@@ -1,8 +1,8 @@
-from .freqnet import *
-from .generating_net import *
 from .parametrized_gaussian import *
+from .resamplers import *
 from .sample_rnn import *
 from .s2s_lstm import *
+from .single_class_mlp import *
 from .tied_autoencoder import *
 from .transformers import *
 from .wavenet import *
